import { Component, Input, OnInit, ViewChild } from '@angular/core'
import { MatTable } from '@angular/material/table'
import { ActivatedRoute } from '@angular/router'
import { Location } from '@angular/common'

import { Mieter } from '../mieter'
import { ZahlungForderung, Saldo, Zahlung, ZahlungRaw } from '../zahlung-forderung'

import { MieterService} from '../mieter.service'
import { ZahlungForderungService} from '../zahlung-forderung.service'
import { MessageService } from '../message.service'
import { MatTableDataSource } from '@angular/material/table'

@Component({
  selector: 'app-mieter-detail',
  templateUrl: './mieter-detail.component.html',
  styleUrls: ['./mieter-detail.component.css']
})
export class MieterDetailComponent implements OnInit {

  mieter: Mieter
  year: string
  zahlungenForderungen : ZahlungForderung[]
  saldo: Saldo
  newZahlung: Zahlung

  displayedColumns: string[] = ["datum_soll", "datum_ist", "betrag_forderung", "betrag_zahlung", "kommentar"]
  dataSource: MatTableDataSource<ZahlungForderung>

  constructor(
    private mieterService: MieterService, 
    private zahlungForderungService: ZahlungForderungService,
    private messageService: MessageService,
    private route: ActivatedRoute,
    private location: Location ) { 
  }

  async getMieter(): Promise<void> {
    const id = +this.route.snapshot.paramMap.get('id')
    try {
      this.mieter = await this.mieterService.getMieter(id)
      await this.getZahlungenForderungen()
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async getZahlungenForderungen(): Promise<void> {
    const id = this.mieter?.id ?? 0
    try {
      this.zahlungenForderungen = await this.zahlungForderungService.getZahlungenForderungenByMieterAndYear(id, +this.year)
      this.dataSource = new MatTableDataSource<ZahlungForderung>(this.zahlungenForderungen)
      this.saldo = await this.zahlungForderungService.getSaldoByMieterAndYear(id, +this.year)
    } catch (err) {
      this.messageService.add(JSON.stringify(err, undefined, 4))
    }
  }

  async loadForYear(): Promise<void> {
    await this.getZahlungenForderungen()
  }

  async insertNewZahlung(newZahlungRaw: ZahlungRaw): Promise<void> {
    try {
      this.messageService.add(`New ZahlungRaw: ${newZahlungRaw.datum_soll}, ${newZahlungRaw.datum_ist}, ${newZahlungRaw.betrag}, ${newZahlungRaw.kommentar}`)
      let newZahlung : Zahlung = {
        'id': 0,
        'mieter': this.mieter.id,
        'datum_soll': newZahlungRaw.datum_soll,
        'datum_ist': newZahlungRaw.datum_ist,
        'betrag': +(newZahlungRaw.betrag.replace(',','.')),
        'kommentar': newZahlungRaw.kommentar
      }
      this.messageService.add(`New Zahlung: ${newZahlung.datum_soll}, ${newZahlung.datum_ist}, ${newZahlung.betrag}, ${newZahlung.kommentar}, ${newZahlung.mieter}`)
      await this.zahlungForderungService.postZahlung(newZahlung)
      await this.getZahlungenForderungen()
    } catch (err) {
      this.messageService.add(`Error when inserted zahlung: ${JSON.stringify(err, undefined, 4)}`)
    }
  }

  async ngOnInit(): Promise<void> {
    this.year = new Date().getFullYear().toString()
    await this.getMieter()
    await this.getZahlungenForderungen()
  }
}
